# -*- coding: utf-8 -*-
'''

BC SALTWATER INTRUSION RISK MAPPING
INTERIM DRAFT REPORT FOR 2019/2020
MARCH, 2020

NAME: buffercoast.py

PURPOSE OF SCRIPT:

Create buffers from coastline for entire coastal BC for bedrock aquifer susceptibility
based on user defined distances

AUTHOR:

Tim Sivak, M.GISA, GIT (license pending)
tim@westernwater.ca

'''
from osgeo import gdal,ogr,osr
from shapely.geometry import shape, mapping
import fiona
import os
import time
import warnings

warnings.filterwarnings("ignore", category=DeprecationWarning)

#global variables

#buffer distance negative so the buffer propogates inward from the coast

x1,h5 = -150, 5 #buffer distance, hazard rating.
x2,h4 = -300, 4
x3,h3 = -900, 3
x4,h2 = -1500, 2
x5,h1 = -3000, 1

path = "D:\\Dropbox (WWA)\\WWAL\\Projects\\2019\\19-098-02VC Saltwater Intrusion Risk Mapping Year 2\\Data Analysis\\data_input\\"
startTime = time.time()

#input landmass
coast = path + "land.shp"

buff1 = path + "land" + str(x1) + ".shp"
buff2 = path + "land" + str(x2) + ".shp"
buff3 = path + "land" + str(x3) + ".shp"
buff4 = path + "land" + str(x4) + ".shp"
buff5 = path + "land" + str(x5) + ".shp"

buff1_f = path + "buff_x1.shp"
buff2_f = path + "buff_x2.shp"
buff3_f = path + "buff_x3.shp"
buff4_f = path + "buff_x4.shp"
buff5_f = path + "buff_x5.shp"

srs = osr.SpatialReference()       ###
srs.SetFromUserInput("EPSG:3005")  ###
ref = srs.ExportToProj4() 

def createBuffer(inputfn, outputBufferfn, bufferDist):
    inputds = ogr.Open(inputfn)
    if not inputds:
        raise IOError("Could not open '%s'"%inputfn)
    inputlyr = inputds.GetLayer()

    shpdriver = ogr.GetDriverByName('ESRI Shapefile')
    if os.path.exists(outputBufferfn):
        shpdriver.DeleteDataSource(outputBufferfn)
    outputBufferds = shpdriver.CreateDataSource(outputBufferfn)
    bufferlyr = outputBufferds.CreateLayer(outputBufferfn, geom_type=ogr.wkbPolygon)
    featureDefn = bufferlyr.GetLayerDefn()

    for feature in inputlyr:
        ingeom = feature.GetGeometryRef()
        geomBuffer = ingeom.Buffer(bufferDist)

        outFeature = ogr.Feature(featureDefn)
        outFeature.SetGeometry(geomBuffer)
        bufferlyr.CreateFeature(outFeature)
        outFeature = None
    
def buffer(inputfn, outputBufferfn, bufferDist):
    createBuffer(inputfn, outputBufferfn, bufferDist)

def saveSHP(poly,out,ref,haz):
    # Define a polygon feature geometry with one attribute
    schema = {
        'geometry': 'Polygon',
        'properties': {'hazard': 'int'},
    }
    
    # Write a new Shapefile
    with fiona.open(out, 'w', 
                    crs=ref,
                    driver='ESRI Shapefile', 
                    schema=schema) as c:
        ## If there are multiple geometries, put the "for" loop here
        c.write({
            'geometry': mapping(poly),
            'properties': {'hazard': haz},
        })

print("Creating buffers...")
print(str(x1) + " m...")   
buffer(coast, buff1, x1)
print(str(x2) + " m...")  
buffer(coast, buff2, x2)
print(str(x3) + " m...")  
buffer(coast, buff3, x3)
print(str(x4) + " m...")  
buffer(coast, buff4, x4)
print(str(x5) + " m...")  
buffer(coast, buff5, x5)

coast = fiona.open(coast)
buffx1 = fiona.open(buff1)
buffx2 = fiona.open(buff2)
buffx3 = fiona.open(buff3)
buffx4 = fiona.open(buff4)
buffx5 = fiona.open(buff5)

coast_feat = coast.next()
buffx1_feat = buffx1.next()
buffx2_feat = buffx2.next()
buffx3_feat = buffx3.next()
buffx4_feat = buffx4.next()
buffx5_feat = buffx5.next()

coast_geom = shape(coast_feat['geometry'])
buffx1_geom = shape(buffx1_feat['geometry'])
buffx2_geom = shape(buffx2_feat['geometry'])
buffx3_geom = shape(buffx3_feat['geometry'])
buffx4_geom = shape(buffx4_feat['geometry'])
buffx5_geom = shape(buffx5_feat['geometry'])

print("Creating difference...")
print(str(x1) + " m...")   
buffx1_final = coast_geom.difference(buffx1_geom)
print(str(x2) + " m...")   
buffx2_final = buffx1_geom.difference(buffx2_geom)
print(str(x3) + " m...")   
buffx3_final = buffx2_geom.difference(buffx3_geom)
print(str(x4) + " m...")   
buffx4_final = buffx3_geom.difference(buffx4_geom)
print(str(x5) + " m...")   
buffx5_final = buffx4_geom.difference(buffx5_geom)

print("Saving shapefiles...")
saveSHP(buffx1_final,buff1_f,ref,5)
saveSHP(buffx2_final,buff2_f,ref,4)
saveSHP(buffx3_final,buff3_f,ref,3)
saveSHP(buffx4_final,buff4_f,ref,2)
saveSHP(buffx5_final,buff5_f,ref,1)

print("Removing temporary files...")

coast = None
buffx1 = None
buffx2 = None
buffx3 = None
buffx4 = None
buffx5 = None
coast_feat = None
buffx1_feat = None
buffx2_feat = None
buffx3_feat = None
buffx4_feat = None
buffx5_feat = None
coast_geom = None
buffx1_geom = None
buffx2_geom = None
buffx3_geom = None
buffx4_geom = None
buffx5_geom = None
buffx1_final = None
buffx2_final = None
buffx3_final = None
buffx4_final = None
buffx5_final = None

del coast
del buffx1
del buffx2
del buffx3
del buffx4
del buffx5
del coast_feat
del buffx1_feat
del buffx2_feat
del buffx3_feat
del buffx4_feat
del buffx5_feat
del coast_geom
del buffx1_geom
del buffx2_geom
del buffx3_geom
del buffx4_geom
del buffx5_geom
del buffx1_final
del buffx2_final
del buffx3_final
del buffx4_final
del buffx5_final

print("Complete.")

endTime = time.time()
print("")
print("Time elapsed: " + str((endTime - startTime)) + " seconds.")