# -*- coding: utf-8 -*-
"""
BC SALTWATER INTRUSION RISK MAPPING
MARCH, 2021

NAME: loss_wells.py

PURPOSE OF SCRIPT:

Calculate loss for unconsolidated and bedrock wells, and water licenses

AUTHOR:

Tim Sivak, MGISA, PGEO
tim@westernwater.ca

"""

import geopandas as gpd
import numpy as np
import pandas as pd

import warnings
import time

# global variables

def calc_Loss(loc_in,area):
    # global variables
    
    startTime = time.time()
    warnings.filterwarnings('ignore', 'GeoSeries.notna')
    input_path = "D:\\Dropbox (WWA)\\WWAL\\Projects\\2019\\19-098-02VC Saltwater Intrusion Risk Mapping Year 2\\Data Analysis\\data_input\\"
    location = "D:\\Dropbox (WWA)\\WWAL\\Projects\\2019\\19-098-02VC Saltwater Intrusion Risk Mapping Year 2\\Data Analysis\\Pilot Areas\\Final Maps_mar\\" + area + "\\"
    
    #input files
    gwells = location + "gwells_parsed.shp"
    extent = location + "extent.shp" #set to an NTS mapsheet
    water_rights = input_path + "water_rights.shp"
    grid = location + "grid.shp"  
    
    #METHOD:
     #1) CREATE UNC AND BED CENTROIDS 
     #2) ASSIGN LOSS CATEGORIES TO EACH WELL AND LICENCE
     #3) CALCULATE TOTAL Q PER RATING CATEGORY PER GRID CELL
     #4) ASSIGN RATING SCHEME
     #5) MERGE CENTROID CATEGORIES FOR UNC AND BED, AND OUTPUT CENTROIDS
        
    #import geodataframes
    water_rights = gpd.GeoDataFrame.from_file(water_rights)
    grid = gpd.GeoDataFrame.from_file(grid)
    extent = gpd.GeoDataFrame.from_file(extent)
    gwells = gpd.GeoDataFrame.from_file(gwells)
    
    #clip water rights to extent
    water_rights = gpd.clip(water_rights, extent)  
        
    #remove water rights points that have no well tag nunbers, therefore only considering PWD
    water_rights.dropna(subset=['WLL_TG_NMR'], inplace=True)
    water_rights['WTN'] = water_rights['WLL_TG_NMR'].astype(int)
    
    water_rights['demand'] = ""
    water_rights['cat_A'] = ""
    water_rights['cat_B'] = ""
    water_rights['cat_C'] = ""
    water_rights['type'] = ""
    water_rights['intended_w'] = ""
        
    gwells['PRPS_S_CD'] = ""
    gwells['cat_A'] = ""
    gwells['cat_B'] = ""
    gwells['cat_C'] = ""
    
    water_rights = water_rights[['WTN','PRPS_S_CD','QUANTITY','QNTTY_NTS','demand','cat_A','cat_B','cat_C','type','intended_w','geometry']] 
    gwells = gwells[['WTN','PRPS_S_CD','demand','cat_A','cat_B','cat_C','type','intended_w','geometry']]
    
    #merge aquifer type to water rights
    
    water_rights['WTN']=water_rights['WTN'].astype(int)
    gwells['WTN']=gwells['WTN'].astype(int)
    
    gwells_type = gwells[['WTN','type']]
    water_rights = pd.merge(gwells_type, water_rights, on='WTN', how='inner')
    
    water_rights['type'] = water_rights['type_x']
    water_rights = water_rights[['WTN','PRPS_S_CD','QUANTITY','QNTTY_NTS','demand','cat_A','cat_B','cat_C','type','intended_w','geometry']] 
    
    #remove gwells that are already in water rights
    water_rights_list = water_rights['WTN'].tolist()  
    gwells = gwells[~gwells['WTN'].isin(water_rights_list)]
    
    #create demand column and convert demand to m3/day
    water_rights['demand'] = np.where(water_rights['QNTTY_NTS']=="m3/year",water_rights['QUANTITY']/365,\
                        np.where(water_rights['QNTTY_NTS']=="m3/sec",water_rights['QUANTITY']*86400, water_rights['QUANTITY']))
        
    water_rights = water_rights[['WTN','PRPS_S_CD','demand','cat_A','cat_B','cat_C','type','intended_w','geometry']]
    
    water_rights = gpd.GeoDataFrame(water_rights, geometry = "geometry")
    #water_rights.to_file(location + "water_rights_loss.shp")
    #gwells.to_file(location + "gwells_loss.shp")
    
    #------------------------------------------------------------------------------------------------------------------------
    # LOSS RATING SCHEME
    
    # CATEGORY A - MINERALIZED OR CONSERVATION USE PURPOSES
    # HIGHEST VALUE CATEGORY
    
    # MINERALIZED WATER
    # 09A - Mineralized Water: Bottling & Dist      
    # 09B
    
    # CONSERVATION
    # 11A
    # 11B - Conservation: Use of Water                  
    # 11C
    
    category_A = ['09A','09B','11A','11B','11C']
    
    condition = ((water_rights['PRPS_S_CD'].isin(category_A)))
    water_rights['cat_A'][condition] = water_rights['demand']
    
    #------------------------------------------------------------------------------------------------------------------------
    #CATEGORY B - DRINKING WATER, IRRIGATION, MANY INDUSTRIAL USE PURPOSES
    #MIDDLE VALUE CATEGORY
    
    #DOMESTIC WELLS
    #already have assigned demands of 2m3/day MINIMUM
    
    #IRRIGATION
    # 03A - Irrigation: Local Provider              
    # 03B - Irrigation: Private

    # WATERWORKS
    # 00A - Waterworks: Local Provider            
    # 00B - Waterworks (other than LP)             
    # 00C - Waterworks: Sales                    
    # 02I35 - Waterworks: Water Delivery 
    
    # INDUSTRIAL
    
    # WSA02 - Camps & Public Facilities #ITEM 2     
    # WSA03 - Commercial Enterprise  #ITEM 3
    # WSA04 - Crop Harvest, Protect & Compost
    # WSA05 - Greenhouse & Nursery      
    # WSA06 - Ice & Snow Making
    # WSA08 - Livestock & Animal    #ITEM 12
    # WSA11 - Lawn, Fairway & Garden 
        
    # 02A - Pulp Mill   
    # 02B - Processing & Mfg: Processing            
    # 02D - Comm. Enterprise: Enterprise #ITEM 3
    # 02E - Pond & Aquaculture   #ITEM 14         
    # 02G - Fresh Water Bottling   
    # 02H - Bulk Shipment for Marine Trans 
      
    # 02I38 - Fish Hatchery    
    
    #GWELLS intended well use::
    #COM - commercial
    #DOM - domestic
    #DWS - drinking water supply
    #IRR - Irrigation
    #NA - not applicable
    #OBS - observation
    #UNK - unknown
    #OP_LP_GEO -
    #OTHER
    #HATCHERY
    #DWSS
    
    gwells_catB = ['COM','DOM','DWS','UNK','IRR','NA','OBS','OP_LP_GEO',\
                   'OTHER','HATCHERY','DWSS']  
    category_B = ['03A','03B','00A','00B','00C','02I35','WSA02',\
                  'WSA03','WSA04','WSA05','WSA06','WSA08','WSA11',\
                  '02A','02B','02D','02E','02G','02H','02I38']
    
    condition = ((water_rights['PRPS_S_CD'].isin(category_B)))
    water_rights['cat_B'][condition] = water_rights['demand']
    
    condition = ((gwells['intended_w'].isin(gwells_catB)))
    gwells['cat_B'][condition] = gwells['demand']
    
    #------------------------------------------------------------------------------------------------------------------------
    # CATEGORY C - OTHER WATER USE PURPOSES
    # LEAST VALUE
    
    # MINING
    # 05A
    # 05B - Mining: Washing Coal                 
    # 05C - Mining: Processing Ore
    # 05D - 
    
    # INDUSTRIAL 
    # 02C - Cooling 
    # WSA07 - Misc Indust #ITEM 13    
    # 02I18 - Heat Exchanger    #ITEM 17   
    # 02I32 - Swimming Pool #ITEM 18  
    # WSA09 - Processing & Manufacturing  
    # WSA12 - Vehicle & Equipment #ITEM 19
    # WSA10 - Well Drill/Transprt Mgmt #ITEM 21 
    # WSA13 - Industrial Waste Mgmt  
    
    # OIL AND GAS
    # 05E - O & G: Hydrlc Frctrg (non-deep GW)         
    # 05F - O & G: Hydrlc Frctrg  (deep GW)  
    # 05G           
    # 05H - O & G: Drilling                             
    # 02I23 - O & G: Oil Fld Inject. (non-deep GW)  
    
    # LAND IMPROVEMENT
    # 04A - Land Improve: General                       
    
    category_C = ['05A','05B','05C','05D','02C','WSA07','02I18',\
                  '02I32','WSA09','WSA12','WSA10','WSA13',\
                  '05E','05F','05G','05H','02I123','04A']
    
    
    condition = ((water_rights['PRPS_S_CD'].isin(category_C)))
    water_rights['cat_A'][condition] = water_rights['demand']
    
    #NORMALIZE AND COMBINE DATAFRAMES
    
    frames = [gwells, water_rights]
    loss_wells = pd.concat(frames)
    
    loss_wells['cat_A']=pd.to_numeric(loss_wells['cat_A'],errors='coerce')
    loss_wells['cat_B']=pd.to_numeric(loss_wells['cat_B'],errors='coerce')
    loss_wells['cat_C']=pd.to_numeric(loss_wells['cat_C'],errors='coerce')
    
    #loss_wells.to_file(location + "licences and wells_loss.shp")
    
    #CREATE CENTROIDS 
    
    loss_UNC = loss_wells.loc[loss_wells['type'] == 'UNC']
    loss_BED = loss_wells.loc[loss_wells['type'] == 'BED']
    
    #Join Unconsolidated centroids
    join_unc = gpd.sjoin(grid, loss_UNC, op='contains', how = 'right')
    sum = join_unc.groupby(['Grid_ID','type']).agg({'cat_A':'sum','cat_B':'sum','cat_C':'sum'})
            
    centroids_unc = grid.copy()
    centroids_unc['geometry'] = centroids_unc['geometry'].centroid      
    centroids_unc = centroids_unc.merge(sum, on='Grid_ID')
                
    #Join bedrock centroids
    join_bed = gpd.sjoin(grid, loss_BED, op='contains', how = 'right')
    sum = join_bed.groupby(['Grid_ID','type']).agg({'cat_A':'sum','cat_B':'sum','cat_C':'sum'})
            
    centroids_bed = grid.copy()
    centroids_bed['geometry'] = centroids_bed['geometry'].centroid      
    centroids_bed = centroids_bed.merge(sum, on='Grid_ID')
    
    centroids_unc['loss_cat'] = ''
    centroids_bed['loss_cat'] = ''
    centroids_unc['loss_rating'] = ''
    centroids_bed['loss_rating'] = ''
    
    def rank_Loss(centroids):  
        #assign loss ranking to each point, based on category and use quantity
        
        #matrix data
        cat1_max = 30
        cat2_min = cat1_max
        cat2_max = 100
        cat3_min = cat2_max
        cat3_max = 300
        cat4_min = cat3_max
        cat4_max = 1000
        cat5_min = cat4_max
    
        centroids['total_Q'] = centroids['cat_A'] + centroids['cat_B'] + centroids['cat_C']
        
        #mineralied water (category A) > drinking water (category B) > land improvement (category C)
            
        #if theres only mining and other industrial wells in the grid, and water quality is not as important
        conditionC = (centroids['cat_C'] > 0)
        
        #if there's any well in the grid with drinking water
        conditionB = (centroids['cat_B'] > 0)
        
        #if theres a mineralized or other high-value source
        conditionA = (centroids['cat_A'] > 0)
             
        centroids['loss_cat'][conditionC] = 'C'   
        centroids['loss_cat'][conditionB] = 'B'
        centroids['loss_cat'][conditionA] = 'A'
        
        centroids_A = centroids.loc[centroids['loss_cat'] == 'A']
        centroids_B = centroids.loc[centroids['loss_cat'] == 'B']
        centroids_C = centroids.loc[centroids['loss_cat'] == 'C']
        
        #mineralized or conservation use purposes
        mask = centroids_A['total_Q'] < cat1_max
        centroids_A['loss_rating'][mask] = 3
        
        mask = (centroids_A['total_Q'] >= cat2_min) & (centroids_A['total_Q'] < cat2_max)
        centroids_A['loss_rating'][mask] = 4
        
        mask = (centroids_A['total_Q'] >= cat3_min) 
        centroids_A['loss_rating'][mask] = 5
          
        #drinking water, irrigation, some industrial use purposes    
        mask = centroids_B['total_Q'] < cat1_max
        centroids_B['loss_rating'][mask] = 2
        
        mask = (centroids_B['total_Q'] >= cat2_min) & (centroids_B['total_Q'] < cat2_max)
        centroids_B['loss_rating'][mask] = 3
        
        mask = (centroids_B['total_Q'] >= cat3_min) & (centroids_B['total_Q'] < cat3_max)
        centroids_B['loss_rating'][mask] = 4
        
        mask = (centroids_B['total_Q'] >= cat4_min)
        centroids_B['loss_rating'][mask] = 5
        
        #land improvement and other remaining industrial use purposes    
        mask = centroids_C['total_Q'] < cat1_max
        centroids_C['loss_rating'][mask] = 1
        
        mask = (centroids_C['total_Q'] >= cat2_min) & (centroids_C['total_Q'] < cat2_max)
        centroids_C['loss_rating'][mask] = 2
        
        mask = (centroids_C['total_Q'] >= cat3_min) & (centroids_C['total_Q'] < cat3_max)
        centroids_C['loss_rating'][mask] = 3
        
        mask = (centroids_C['total_Q'] >= cat4_min) & (centroids_C['total_Q'] < cat4_max)
        centroids_C['loss_rating'][mask] = 4
        
        mask = (centroids_C['total_Q'] >= cat5_min)
        centroids_C['loss_rating'][mask] = 5
        
        frames = [centroids_A, centroids_B, centroids_C]
        
        centroids['loss_rating']=pd.to_numeric(centroids['loss_rating'],errors='coerce')
        
        centroids = pd.concat(frames)
        
        return centroids
        
    centroids_unc = rank_Loss(centroids_unc)
    centroids_bed = rank_Loss(centroids_bed)
    
    #convert to integer
    centroids_unc['loss_rating']=pd.to_numeric(centroids_unc['loss_rating'],errors='coerce')
    centroids_bed['loss_rating']=pd.to_numeric(centroids_bed['loss_rating'],errors='coerce')
    
    centroids_bed.to_file(location + "loss_centroids_bed.shp")  
    centroids_unc.to_file(location + "loss_centroids_unc.shp")
    
    endTime = time.time()
    print("")
    print("Time elapsed: " + str((endTime - startTime)) + " seconds.")