# -*- coding: utf-8 -*-
"""
Created on Thu Mar 26 08:32:34 2020

BC SALTWATER INTRUSION RISK MAPPING
MARCH, 2021

NAME: main.py

PURPOSE OF SCRIPT:

- iterate through pumping threat, aquifer susceptibility, and loss for mapping extents (areas of interest) located within variable "folder path" 

AUTHOR:

Tim Sivak, MGISA, PGEO
tim@westernwater.ca
"""

# custom functions
import geopandas as gpd
from pumping_threat_funct import calc_PumpThreat
from aquifer_susceptibility_funct import calc_AqSusceptUNC
from loss_funct import calc_Loss

import os

#location of mapping extent "extent.shp" for area of interest
folder_path = "D:\\Dropbox (WWA)\\WWAL\\Projects\\2019\\19-098-02VC Saltwater Intrusion Risk Mapping Year 2\\Data Analysis\\Pilot Areas\\Final Maps_mar\\"

#location of model input datasets
input_path = "D:\\Dropbox (WWA)\\WWAL\\Projects\\2019\\19-098-02VC Saltwater Intrusion Risk Mapping Year 2\\Data Analysis\\data_input\\"

#get locations
dirlist = [ item for item in os.listdir(folder_path) if os.path.isdir(os.path.join(folder_path, item)) ]
print(dirlist)

print("Importing global GWELLS layer...")
gwells = input_path + "coastal_wells_point.shp"
gwells = gpd.GeoDataFrame.from_file(gwells)

i = 0

#PUMPING THREAT

for x in dirlist:
    location = dirlist[i]
    print("Calculating pumping threat in area: " + location)
    calc_PumpThreat(gwells,folder_path + "\\" + location,location)

    i += 1

i = 0

#AQUIFER SUSCEPTIBILITY

for x in dirlist:
    location = dirlist[i]
    print("Calculating aquifer susceptibility in area: " + location)
    calc_AqSusceptUNC(folder_path + "\\" + location,location)
    i += 1

i = 0

#LOSS

for x in dirlist:
    location = dirlist[i]
    print("Calculating loss in area: " + location)
    calc_Loss(folder_path + "\\" + location,location)
    i += 1

i = 0
